package cn.com.qmth.examcloud.core.basic.api.provider;

import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.api.OrgCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.OrgBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetOrgListByNameLikeReq;
import cn.com.qmth.examcloud.core.basic.api.request.GetOrgReq;
import cn.com.qmth.examcloud.core.basic.api.request.SaveOrgReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetOrgListByNameLikeResp;
import cn.com.qmth.examcloud.core.basic.api.response.GetOrgResp;
import cn.com.qmth.examcloud.core.basic.api.response.SaveOrgResp;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.OrgEntity;
import cn.com.qmth.examcloud.core.basic.service.impl.OrgService;
import io.swagger.annotations.ApiOperation;

@Transactional
@RestController
@RequestMapping("${$rmp.cloud.basic}" + "org")
public class OrgCloudServiceProvider extends ControllerSupport implements OrgCloudService {

	private static final long serialVersionUID = -7858439296389761341L;

	@Autowired
	private OrgService orgService;

	@Autowired
	private OrgRepo orgRepo;

	@ApiOperation(value = "保存机构")
	@PostMapping("saveOrg")
	@Override
	public SaveOrgResp saveOrg(@RequestBody SaveOrgReq orgReq) {
		OrgEntity org = orgRepo.findByRootIdAndCode(orgReq.getRootOrgId(), orgReq.getOrgCode());
		if (org == null) {
			org = new OrgEntity();
		}
		org = new OrgEntity();
		org.setName(orgReq.getOrgName());
		org.setCode(orgReq.getOrgCode());
		org.setParentId(orgReq.getRootOrgId());
		org.setRootId(orgReq.getRootOrgId());
		org.setEnable(true);
		org = orgService.save(org);

		SaveOrgResp orgResp = new SaveOrgResp();
		orgResp.setId(org.getId());
		orgResp.setName(org.getName());
		return orgResp;
	}

	@ApiOperation(value = "按机构名称模糊查询机构列表")
	@PostMapping("getOrgListByNameLike")
	@Override
	public GetOrgListByNameLikeResp getOrgListByNameLike(@RequestBody GetOrgListByNameLikeReq req) {
		String orgName = req.getOrgName();
		String rootOrgId = req.getRootOrgId();

		List<OrgEntity> list = Lists.newArrayList();
		if (StringUtils.isNotBlank(orgName)) {
			list = orgRepo.findByRootIdAndNameLike(Long.parseLong(rootOrgId), orgName);
		}

		GetOrgListByNameLikeResp resp = new GetOrgListByNameLikeResp();
		List<OrgBean> orgList = Lists.newArrayList();
		resp.setOrgList(orgList);

		if (CollectionUtils.isNotEmpty(list)) {
			for (OrgEntity curOrg : list) {
				OrgBean orgBean = new OrgBean();
				orgBean.setId(curOrg.getId());
				orgBean.setName(curOrg.getName());
				orgBean.setParentId(curOrg.getParentId());
				orgBean.setRootId(curOrg.getRootId());
				orgList.add(orgBean);
			}
		}

		return resp;
	}

	@ApiOperation(value = "按机构名称模糊查询机构列表")
	@PostMapping("getOrg")
	@Override
	public GetOrgResp getOrg(@RequestBody GetOrgReq req) {
		Long orgId = req.getOrgId();
		String orgCode = req.getOrgCode();
		Long rootOrgId = req.getRootOrgId();

		if (null == orgId && StringUtils.isBlank(orgCode)) {
			throw new StatusException("B-150000", "orgId,orgCode不能都为空");
		}

		if (null != orgId && StringUtils.isNotBlank(orgCode)) {
			throw new StatusException("B-150001", "orgId,orgCode不能都不为空");
		}

		OrgEntity org = null;
		if (null != orgId) {
			org = orgRepo.findOne(orgId);
		} else if (StringUtils.isNotBlank(orgCode)) {
			if (null == rootOrgId) {
				throw new StatusException("B-150001", "rootOrgId is null");
			}
			org = orgRepo.findByRootIdAndCode(rootOrgId, orgCode);
		}

		if (null == org) {
			throw new StatusException("B-150001", "机构不存在");
		}

		OrgBean orgBean = new OrgBean();
		orgBean.setId(org.getId());
		orgBean.setName(org.getName());
		orgBean.setCode(org.getCode());
		orgBean.setParentId(org.getParentId());
		orgBean.setRootId(org.getRootId());
		orgBean.setEnable(org.getEnable());

		GetOrgResp resp = new GetOrgResp();
		resp.setOrg(orgBean);
		return resp;
	}
}
